/**
 *
 * ScrollerBase
 * 提供在移动端支持滚动的容器,支持手势惯性和弹性,利用CSS3实现.
 * 支持IOS和Android
 *
 * Created by TonyJiang on 16/5/7.
 */
import React from 'react';

import ScrollBar from '../common/ScrollBar';

//贝塞尔曲线
const cubic = {

    //超出边界减速
    quadratic: 'cubic-bezier(0.25, 0.46, 0.45, 0.94)',

    //回弹曲线
    outCirc: 'cubic-bezier(0.075, 0.82, 0.165, 1)',

    //惯性曲线
    outCubic: 'cubic-bezier(0.165, 0.84, 0.44, 1)',

    //常规曲线
    circular: 'cubic-bezier(0.1, 0.57, 0.1, 1)'
};

class ScrollerBase extends React.Component {

    static propTypes = {
        onScroll: React.PropTypes.func
    };

    static defaultProps = {
        onScroll: () => {
        },
        onReachTop: () => {
        },
        onLeaveTop: () => {
        },
        onReachBottom: () => {
        },
        onLeaveBottom: () => {
        }

    };

    _state = {
        reachTop: false,
        reachBottom: false,
        deltaY: 0,
        startDeltaY: 0,

        lastScrollHeight: 0,
        lastClientHeight: 0

    };

    _checkTimer = 0;

    constructor(props, context) {
        super(props, context);
    }

    componentWillMount() {
        this._randomClassName = 'scroller_' + new Date().getTime();
        this._hasScrollBar = mutants.env.os != mutants.env.constant.os.ios;
    }

    componentDidMount() {
        //渲染完毕后,处理一次当前scroll的状态, setTimeout保证不会因为父类没有完成DidMount
        setTimeout(()=> {
            this._eventHandler.scroll();
        }, 1);

        this._method.startCheckContentHeightUpdated();

    }

    componentWillUnmount(){
        this._method.stopCheckContentHeightUpdated();
    }

    componentDidUpdate() {
        const {scrollTop, offsetHeight, scrollHeight} = this.refs.domNode;

        if (this._state.reachTop && scrollTop != 0) {
            this._state.reachTop = false;
            this._eventHandler.leaveTop();
        } else if (!this._state.reachTop && scrollTop == 0) {
            this._state.reachTop = true;
            this._eventHandler.reachTop();
        }


        if (this._state.reachBottom && scrollTop + offsetHeight < scrollHeight) {
            this._state.reachBottom = false;
            this._eventHandler.leaveBottom();

        } else if (!this._state.reachBottom && scrollTop + offsetHeight == scrollHeight) {
            this._state.reachBottom = true;
            this._eventHandler.reachBottom();
        }

        if (this._hasScrollBar) {
            this._method.updateScrollBar();
        }

        //派发 onScroll 事件
        this.props.onScroll({
            scrollTop, offsetHeight, scrollHeight
        });
    }

    scrollTo(deltaY, duration = 0) {
        this.refs.domNode.scrollTop = deltaY;
    }

    _getStyles() {

        const styles = {
            height: '100%',
            width: '100%'
        };

        const touchOverrideStyles = {
            WebkitOverflowScrolling: 'touch',
            overflowY: 'scroll'
        };

        if (mutants.env.os != mutants.env.constant.os.ios) {
            touchOverrideStyles.WebkitScrollbar = ''
        }

        return Object.assign(styles, this.props.style, touchOverrideStyles);

    }

    _method = {
        updateScrollBar: () => {
            if (!this._hasScrollBar) {
                return;
            }
            const {scrollTop, offsetHeight, scrollHeight} = this.refs.domNode;
            const {scrollBar} = this.refs;

            scrollBar.show();
            scrollBar.scrollTo({deltaY: scrollTop, contentHeight: scrollHeight, clientHeight: offsetHeight});

            if (!this._state.touching) {
                clearTimeout(this._hideScrollbarUI);
                this._hideScrollbarUI = setTimeout(() => {
                    scrollBar.hide();
                }, 500);
            }

        },

        startCheckContentHeightUpdated: () => {
            this._checkTimer = setInterval(() => {
                this._method.checkContentHeightUpdated();
            }, 16.7);
        },

        stopCheckContentHeightUpdated: () => {
            clearInterval(this._checkTimer);
        },

        //检测内容高度是否发生变化, 然后重新判断reach
        checkContentHeightUpdated: () => {
            const {lastScrollHeight, lastClientHeight} = this._state;
            const {scrollHeight, clientHeight} = this.refs.domNode;

            //如果视窗大小和内容高度没有发生变化, 直接返回
            if(scrollHeight == lastScrollHeight && clientHeight == lastClientHeight){
                return;
            }

            this._eventHandler.scroll(true);

            this._state.lastScrollHeight = scrollHeight;
            this._state.lastClientHeight = clientHeight;

        }

    }

    _eventHandler = {
        touchStart: (e) => {
            this._state.touching = true;
            if (this._hasScrollBar) {
                this.refs.scrollBar.show();
            }
        },

        touchEnd: () => {
            this._state.touching = false;
            if (this._hasScrollBar) {
                this.refs.scrollBar.hide();
            }
        },

        scroll: (isCheck = false) => {
            const {scrollTop, offsetHeight, scrollHeight} = this.refs.domNode;

            //更新scrollbar位置
            this._method.updateScrollBar();

            const topLine = scrollTop;
            const bottomLine = scrollTop + offsetHeight;

            //检测是否需要派发 onReachTop 事件
            //本来应该是==0 但是由于ios有弹性,持续上拉会变成负数,所以兼容ios
            if (topLine <= 0) {
                if (!this._state.reachTop) {
                    this._state.reachTop = true;
                    this._eventHandler.reachTop();
                }

            }else{
                if (this._state.reachTop) {
                    this._state.reachTop = false;
                    this._eventHandler.leaveTop();
                }
            }

            //检测是否需要派发 onReachBottom 事件
            //本来应该是==scrollHeight 但是由于ios有弹性,持续下拉会持续增加,所以兼容ios
            //scrollHeight - 2 是因为在浏览器中, rem转px时, 会有小数问题, 所以导致scrollHeight比scrollTop+contentHeight大1~2, 设置2保险;
            if (bottomLine >= scrollHeight - 2) {

                if (!this._state.reachBottom) {
                    this._state.reachBottom = true;
                    this._eventHandler.reachBottom();
                }

            } else {

                if (this._state.reachBottom) {
                    this._state.reachBottom = false;
                    this._eventHandler.leaveBottom();
                }

            }

            //派发 onScroll 事件
            if(!isCheck){
                this.props.onScroll({
                    scrollTop, offsetHeight, scrollHeight
                });
            }

        },

        reachTop: () => {
            this.props.onReachTop();
        },

        leaveTop: () => {
            this.props.onLeaveTop();
        },

        reachBottom: () => {
            this.props.onReachBottom();
        },

        leaveBottom: () => {
            this.props.onLeaveBottom();
        }
    }

    render() {
        const styles = this._getStyles();


        const scrollBarStyles = `
            .${this._randomClassName}::-webkit-scrollbar{
                display:none;
            }
        `;

        //根据环境判断是否需要自己实现滚动条效果, 安卓4.0中, webkit-overflow-scrolling:touch 没有显示滚动条
        let scrollBar = undefined;
        if (!this._hasScrollBar) {
            scrollBar = '';
        } else {
            scrollBar = (
                <ScrollBar
                    ref="scrollBar"
                />
            );
        }

        return (
            <div
                ref="domNode"
                style={styles}
                onScroll={()=>this._eventHandler.scroll()}
                onTouchStart={this._eventHandler.touchStart}
                onTouchEnd={this._eventHandler.touchEnd}
                className={this._randomClassName}
            >
                {this.props.children}
                <style>{scrollBarStyles}</style>
                {scrollBar}

            </div>
        );

    }

}

export default ScrollerBase;